@@ -5,8 +5,8 @@
 #' @param data A \code{data.frame} with data.
 #' @param tree A phylogenetic tree of class \code{pylo}.
 #' @param cor_fun A function that creates a \code{corStruct} object, typically
-#'   one of the cor function from the \code{ape}, such as \code{corBrownian},
-#'   \code{corPagel} etc.
+#'   one of the cor functions from the \code{ape} package, such as
+#'   \code{corBrownian}, \code{corPagel} etc.
 #' @param order Causal order of the included variable, given as a character
 #'   vector. This is used to determine which variable should be the dependent
 #'   in the dsep regression equations. If left unspecified, the order will be
@@ -17,6 +17,8 @@
 #' @param parallel An optional vector containing the virtual connection
 #'   process type for running the chains in parallel (such as \code{"SOCK"}).
 #'   A cluster is create using the \code{parallel} package.
+#' @param na.rm Should rows that contain missing values be dropped from the data
+#'   as necessary (with a message)?
 #'
 #' @return A phylopath object, with the following components:
 #'  \describe{
@@ -37,22 +39,43 @@
 #'   # And the summary gives statistics to compare the models:
 #'   summary(p)
 #'
-phylo_path <- function(models, data, tree, order = NULL,
-                       cor_fun = ape::corPagel, parallel = NULL) {
+phylo_path <- function(models, data, tree, cor_fun = ape::corPagel,
+                       order = NULL, parallel = NULL, na.rm = TRUE) {
   cor_fun <- match.fun(cor_fun)
   # Check if all models have the same number of nodes
   var_names <- lapply(models, colnames)
   if (length(models) > 1 &
       (stats::var(lengths(models)) != 0 |
        any(lengths(sapply(var_names[-1], setdiff, var_names[[1]])) != 0))) {
-    stop('All causal models need to include the same variables.', call. = FALSE)
+    stop('All causal models need to include the same variables. Combined, your
+         models include the following variables:\n',
+         paste(sort(unique(unlist(var_names))), collapse = '\n'),
+         call. = FALSE)
+  }
+  data <- data[, unique(unlist(var_names))]
+  # Check NAs and if models and tree line up
+  if ('tbl_df' %in% class(data)) {
+    data <- as.data.frame(data)
+  }
+  if (anyNA(data)) {
+    if (na.rm) {
+      NAs <- which(apply(data, 1, anyNA))
+      message(length(NAs), ' rows were dropped because they contained NA values.')
+      data <- data[-NAs, ]
+    } else {
+      stop('NA values were found in the variables of interest.', call. = FALSE)
+    }
+  }
+  if (length(setdiff(rownames(data), tree$tip.label)) > 0) {
+    stop('Make sure that species in your data have rownames that are exactly matched by name with tips in the tree.')
+  }
+  if (length(tree$tip.label) > nrow(data)) {
+    tree <- ape::drop.tip(tree, setdiff(tree$tip.label, rownames(data)))
+    message('Pruned tree to drop species not included in dat.')
   }
   if (is.null(names(models))) {
     names(models) <- LETTERS[1:length(models)]
   }
-  if ('tbl_df' %in% class(data)) {
-    data <- as.data.frame(data)
-  }
   if (is.null(order)) {
     order <- find_consensus_order(models)
   }
@@ -70,9 +93,19 @@ phylo_path <- function(models, data, tree, order = NULL,
   } else {
     cl <- NULL
   }
-  dsep_models <- pbapply::pblapply(f_list, function(x) {
+  dsep_models_runs <- pbapply::pblapply(f_list, function(x) {
     gls2(x, data = data, tree = tree, cor_fun = cor_fun)
   }, cl = cl)
+  # Produce appropriate error if needed
+  errors <- purrr::map(dsep_models_runs, 'error')
+  purrr::map2(errors, f_list,
+              ~if(!is.null(.x))
+                stop(paste('Fitting the following model:\n   ',
+                           Reduce(paste, deparse(f_list[[1]])),
+                           '\nproduced this error:\n   ', .x),
+                     call. = FALSE))
+  # Otherwise collect models and move on.
+  dsep_models <- purrr::map(dsep_models_runs, 'result')
   dsep_models <- purrr::map(formulas, ~dsep_models[match(.x, f_list)])
 
   d_sep <- purrr::map2(formulas, dsep_models,
